const {Accessor} = require("./Accessor");
const {md5} = require("./utils");

class MysqlAccessor extends Accessor {
	#config = {
		// connection
	};

	// use keys like:
	// - <table name>/<key field name>/<key field value>/<field name> (one field) or
	// - <table name>/<key field name>/<key field value> (all fields)

	get(key, onResult) {
		const [table, keyFieldName, keyFieldValue, fieldName] = key.split("/");
		this.#config.connection.query(
			"SELECT " + (fieldName || "*") + " FROM " + table + " WHERE " + keyFieldName + "=?",
			[keyFieldValue],
			(error, results, fields) => {
				if (error) {
					onResult && onResult(false, "db error " + error.code);
					return;
				}
				if (!results.length) {
					onResult && onResult(false, "unknown key");
					return;
				}
				if (results.length > 1) {
					onResult && onResult(false, "bad key");
					return;
				}
				onResult && onResult(true, fieldName === undefined ? results[0] : results[0][fieldName]);
			}
		);
	}

	set(key, value, onResult) {
		const [table, keyFieldName, keyFieldValue, fieldName] = key.split("/");
		const names = [keyFieldName];
		const values = [keyFieldValue];
		if (fieldName !== undefined) {
			names.push(fieldName);
			values.push(value);
		} else {
			for (const x in value) {
				if (x === keyFieldName) {
					continue;
				}
				names.push(x);
				values.push(value[x]);
			}
		}
		this.#config.connection.query(
			"REPLACE INTO " + table + " (" + names.join(",") + ") VALUES (" + names.map(() => "?").join(",") + ")",
			values,
			(error, results, fields) => {
				if (error) {
					onResult && onResult(false, "db error " + error.code);
					return;
				}
				onResult && onResult(true);
			}
		);
	}

	remove(key, onResult) {
		const [table, keyFieldName, keyFieldValue] = key.split("/");
		this.#config.connection.query(
			"DELETE FROM " + table + " WHERE " + keyFieldName + "=?",
			[keyFieldValue],
			(error, results, fields) => {
				if (error) {
					onResult && onResult(false, "db error " + error.code);
					return;
				}
				onResult && onResult(true);
			}
		);
	}

	hash(key, value) {
		return md5(JSON.stringify({data: value}));
	}

	constructor(config) {
		super();

		Object.assign(this.#config, config);
	}
}

module.exports = {MysqlAccessor};
